!     Last change:  ZZZ  16 Sep 2006   12:39 pm
! Solves problem for controls given a set of parameters, par
! -- starts with guesses for all value functions and iterates
! -- streamlined to avoid subroutines



subroutine controls(nparsv,par)



USE mod_valfun   ! includes mod_grid
USE mod_param    ! includes mod_grid


IMPLICIT NONE


integer, intent(in):: nparsv
double precision, intent(in):: par(nparsv)


double precision, parameter:: pi=3.14159265358979


integer:: i,iter
!integer:: maxU0ind,maxU1ind,maxW1ind(nw)


double precision:: oneminusF(nw),intW0gtRes(nw),intW1gtRes(nw)
double precision:: U0old,U1old,W0old(nw),W1old(nw)
double precision:: U0vec(nw),U1vec(nw),W1vec(nw)
double precision:: diff0(nw),diff1(nw),mse0,mse1,mse


double precision:: tmp(1)


! parameters determining maximum iterations and tolerance for convergence in value functions
integer:: maxiter=5000
double precision:: conv_tol=1.0d-2 


external sumtop


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


! 0.1) re-name parameters


lam0  = par(1)
lam1  = par(2)
delta = par(3)
r     = par(4)
z     = par(5)
muw   = par(6)
sigw  = par(7)
eps   = par(8)
Pd    = par(9)
Pu    = par(10)
Pf    = par(11)


! 0.2) Get distribution function, fw  !!!! UPDATE THIS BASED ON BEST APPROXIMATION METHODS


fw = dexp(-((dlog(wgrid) - muw)**2) / (2.0d0*sigw**2)) / (wgrid*sigw*dsqrt(2.0d0*pi))



! 0.3) compute 1-F(w) for all values of w using distribution function fw(w)  


call sumtop(fw,nw,oneminusF)



! 0.4) fix fw(w) so it sums to one


fw = fw / oneminusF(1)          


oneminusF = oneminusF / oneminusF(1)         



! 1) Initial guesses for U0, U1, W0 and W1 value functions


U0 = z/r
U1 = z/r
W0 = wgrid/r
W1 = (wgrid + eps)/r



!!!!!!!!!!!!  Iterate to convergence on W0(w) and W1(w)   !!!!!!!!!!!!


do iter=1,maxiter


!  write(6,*) 'iteration ',iter
  U0old = U0
  U1old = U1
  W0old = W0
  W1old = W1


  ! calculate sums/integrals of f(w)*Wj(w) from each value of w to max(w) -- returns a vector of these sum/integrals


  call sumtop(W0old*fw,nw,intW0gtRes)
  call sumtop(W1old*fw,nw,intW1gtRes)


! 2a) Solve problem for U0



  U0vec = (z + lam0*intW0gtRes + (1.0d0 - lam0*oneminusF)*U0old)/(1.0d0+r)
  U0 = maxval(U0vec)



! 2b) Solve problem for U1


  U1vec = (z + lam0*intW1gtRes + (1.0d0 - lam0*oneminusF)*(Pd*U0old + (1.0d0-Pd)*U1old))/(1.0d0+r)
  U1 = maxval(U1vec)



! 3a) Solve problem for W0


  W0 = (wgrid + lam1*intW0gtRes + delta*U0old + (1.0d0 - delta - lam1*oneminusF)*(Pu*W1old + (1.0d0 - Pu)*W0old))/(1.0d0 + r)



! 3b) Solve problem for W1



  do i=1,nw
    W1vec =  (wgrid(i)+eps + lam1*(Pf*intW0gtRes + (1.0d0-Pf)*intW1gtRes(i)) + delta*(Pf*U0old + (1.0d0-Pf)*U1old)  &
        &  + (1.0d0 - delta - lam1*(Pf*oneminusF + (1.0d0-Pf)*oneminusF(i)))*W1old(i) ) / (1.0d0 + r)


    W1(i) = maxval(W1vec)


        tmp = wgrid(maxloc(W1vec))
        Re1(i) = tmp(1)
  enddo
  



! 4) Check for convergence - compute mean squared error for W0 and W1ma


  diff0 = W0-W0old
  diff1 = W1-W1old
  
  mse0  = dot_product(diff0,diff0)/nw
  mse1  = dot_product(diff1,diff1)/nw
  mse   = (mse0+mse1)/2.0d0


  write(6,*) iter,'MSE 0=',mse0,'  MSE1=',mse1,' MSE=',mse


124 format('U0  = ',f10.2,'   U1  = ',f10.2)
125 format('Ru0 = ',f10.2,'   Ru1 = ',f10.2)
126 format(i4,f10.2,f12.2,2f14.2)


  if (mse .lt. conv_tol) then


    write(6,*) 'Converged with error: ',mse


        write(11,*) 'Converged with error: ',mse
        write(11,*) 'Number of iterations: ',iter
    write(11,*) ' '
        write(11,*) ' '
        write(11,*) 'lam0  = ',lam0
        write(11,*) 'lam1  = ',lam1
        write(11,*) 'delta = ',delta
        write(11,*) 'r     = ',r
        write(11,*) 'z     = ',z
        write(11,*) 'muw   = ',muw
        write(11,*) 'sigw  = ',sigw
        write(11,*) 'eps   = ',eps
        write(11,*) 'Pd    = ',Pd
        write(11,*) 'Pu    = ',Pu
        write(11,*) 'Pf    = ',Pf
    write(11,*) ' '
        write(11,*) ' '
        write(11,*) 'Final Value functions and Reservation Wages'
        write(11,*) ' '
    write(11,124) U0,U1
    write(11,125) wgrid(maxloc(U0vec)),wgrid(maxloc(U1vec))
        write(11,*) ' '


    write(11,*) 'i        w         Re1(w)       W0(w)         W1(w)'
    write(11,*) '-----------------------------------------------------'
    do i=1,nw
      write(11,126) i,wgrid(i),Re1(i),W0(i),W1(i)
    enddo


    EXIT


  endif



enddo


if (iter > maxiter)  write(6,*) 'Maximum iterations reached ',maxiter
 
END SUBROUTINE





! subroutine to compute the sum of x(i) from all possible i values to imax
! -- Returns: vector sumx of same length as x (length=n)
! -- Generic: uses no modules


SUBROUTINE sumtop(x,n,sumx)


IMPLICIT NONE


integer, intent(in):: n
double precision, intent(in):: x(n)
double precision, intent(out):: sumx(n)


integer:: i




sumx = 0.0d0
sumx(n) = x(n)


do i=1,n-1 
  sumx(n-i)  = x(n-i) + sumx(n-i+1)
enddo


END SUBROUTINE

